# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qbitwallet.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    0,
    '',
    'qbitwallet.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import qbitcoin.generated.qbit_pb2 as qbit__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10qbitwallet.proto\x12\x03qrl\x1a\nqbit.proto\"9\n\x10\x41\x64\x64NewAddressReq\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x15\n\rhash_function\x18\x02 \x01(\t\"A\n\x11\x41\x64\x64NewAddressResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\t\"]\n\x1a\x41\x64\x64NewAddressWithSlavesReq\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x18\n\x10number_of_slaves\x18\x02 \x01(\x04\x12\x15\n\rhash_function\x18\x03 \x01(\t\"%\n\x15\x41\x64\x64\x41\x64\x64ressFromSeedReq\x12\x0c\n\x04seed\x18\x01 \x01(\t\"F\n\x16\x41\x64\x64\x41\x64\x64ressFromSeedResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\t\"\x12\n\x10ListAddressesReq\"C\n\x11ListAddressesResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x11\n\taddresses\x18\x03 \x03(\t\"#\n\x10RemoveAddressReq\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"0\n\x11RemoveAddressResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"\"\n\x0fValidAddressReq\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\">\n\x10ValidAddressResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\r\n\x05valid\x18\x03 \x01(\t\"&\n\x10\x45ncryptWalletReq\x12\x12\n\npassphrase\x18\x01 \x01(\t\"0\n\x11\x45ncryptWalletResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"%\n\x0fUnlockWalletReq\x12\x12\n\npassphrase\x18\x01 \x01(\t\"/\n\x10UnlockWalletResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"\x0f\n\rLockWalletReq\"-\n\x0eLockWalletResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"&\n\x13GetRecoverySeedsReq\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"V\n\x14GetRecoverySeedsResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07hexseed\x18\x03 \x01(\t\x12\x10\n\x08mnemonic\x18\x04 \x01(\t\"\x12\n\x10GetWalletInfoReq\"n\n\x11GetWalletInfoResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\r\x12\x15\n\raddress_count\x18\x04 \x01(\x04\x12\x14\n\x0cis_encrypted\x18\x05 \x01(\x08\"\x8c\x01\n\x13RelayTransferTxnReq\x12\x14\n\x0c\x61\x64\x64resses_to\x18\x01 \x03(\t\x12\x0f\n\x07\x61mounts\x18\x02 \x03(\x04\x12\x0b\n\x03\x66\x65\x65\x18\x03 \x01(\x04\x12\x16\n\x0emaster_address\x18\x04 \x01(\t\x12\x16\n\x0esigner_address\x18\x05 \x01(\t\x12\x11\n\tots_index\x18\x06 \x01(\x04\"h\n\x1aRelayTransferTxnBySlaveReq\x12\x14\n\x0c\x61\x64\x64resses_to\x18\x01 \x03(\t\x12\x0f\n\x07\x61mounts\x18\x02 \x03(\x04\x12\x0b\n\x03\x66\x65\x65\x18\x03 \x01(\x04\x12\x16\n\x0emaster_address\x18\x04 \x01(\t\"u\n\x12RelayMessageTxnReq\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0b\n\x03\x66\x65\x65\x18\x02 \x01(\x04\x12\x16\n\x0emaster_address\x18\x03 \x01(\t\x12\x16\n\x0esigner_address\x18\x04 \x01(\t\x12\x11\n\tots_index\x18\x05 \x01(\x04\"Q\n\x19RelayMessageTxnBySlaveReq\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0b\n\x03\x66\x65\x65\x18\x02 \x01(\x04\x12\x16\n\x0emaster_address\x18\x03 \x01(\t\"\xc5\x01\n\x10RelayTokenTxnReq\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05owner\x18\x03 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\x04 \x01(\x04\x12\x11\n\taddresses\x18\x05 \x03(\t\x12\x0f\n\x07\x61mounts\x18\x06 \x03(\x04\x12\x0b\n\x03\x66\x65\x65\x18\x07 \x01(\x04\x12\x16\n\x0emaster_address\x18\x08 \x01(\t\x12\x16\n\x0esigner_address\x18\t \x01(\t\x12\x11\n\tots_index\x18\n \x01(\x04\"\xa1\x01\n\x17RelayTokenTxnBySlaveReq\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05owner\x18\x03 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\x04 \x01(\x04\x12\x11\n\taddresses\x18\x05 \x03(\t\x12\x0f\n\x07\x61mounts\x18\x06 \x03(\x04\x12\x0b\n\x03\x66\x65\x65\x18\x07 \x01(\x04\x12\x16\n\x0emaster_address\x18\x08 \x01(\t\"\xa7\x01\n\x18RelayTransferTokenTxnReq\x12\x14\n\x0c\x61\x64\x64resses_to\x18\x01 \x03(\t\x12\x14\n\x0ctoken_txhash\x18\x02 \x01(\t\x12\x0f\n\x07\x61mounts\x18\x03 \x03(\x04\x12\x0b\n\x03\x66\x65\x65\x18\x04 \x01(\x04\x12\x16\n\x0emaster_address\x18\x05 \x01(\t\x12\x16\n\x0esigner_address\x18\x06 \x01(\t\x12\x11\n\tots_index\x18\x07 \x01(\x04\"\x83\x01\n\x1fRelayTransferTokenTxnBySlaveReq\x12\x14\n\x0c\x61\x64\x64resses_to\x18\x01 \x03(\t\x12\x14\n\x0ctoken_txhash\x18\x02 \x01(\t\x12\x0f\n\x07\x61mounts\x18\x03 \x03(\x04\x12\x0b\n\x03\x66\x65\x65\x18\x04 \x01(\x04\x12\x16\n\x0emaster_address\x18\x05 \x01(\t\"\x8b\x01\n\x10RelaySlaveTxnReq\x12\x11\n\tslave_pks\x18\x01 \x03(\x0c\x12\x14\n\x0c\x61\x63\x63\x65ss_types\x18\x02 \x03(\r\x12\x0b\n\x03\x66\x65\x65\x18\x03 \x01(\x04\x12\x16\n\x0emaster_address\x18\x04 \x01(\t\x12\x16\n\x0esigner_address\x18\x05 \x01(\t\x12\x11\n\tots_index\x18\x06 \x01(\x04\"g\n\x17RelaySlaveTxnBySlaveReq\x12\x11\n\tslave_pks\x18\x01 \x03(\x0c\x12\x14\n\x0c\x61\x63\x63\x65ss_types\x18\x02 \x03(\r\x12\x0b\n\x03\x66\x65\x65\x18\x03 \x01(\x04\x12\x16\n\x0emaster_address\x18\x04 \x01(\t\"N\n\x0cRelayTxnResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12!\n\x02tx\x18\x03 \x01(\x0b\x32\x15.qrl.PlainTransaction\"C\n\x13\x43hangePassphraseReq\x12\x15\n\roldPassphrase\x18\x01 \x01(\t\x12\x15\n\rnewPassphrase\x18\x02 \x01(\t\"3\n\x14\x43hangePassphraseResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"+\n\x18TransactionsByAddressReq\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"z\n\x19TransactionsByAddressResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12/\n\x11mini_transactions\x18\x03 \x03(\x0b\x32\x14.qrl.MiniTransaction\x12\x0f\n\x07\x62\x61lance\x18\x04 \x01(\x04\"`\n!PaginatedTransactionsByAddressReq\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x15\n\ritem_per_page\x18\x02 \x01(\x04\x12\x13\n\x0bpage_number\x18\x03 \x01(\x04\"\x83\x01\n\"PaginatedTransactionsByAddressResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12/\n\x11mini_transactions\x18\x03 \x03(\x0b\x32\x14.qrl.MiniTransaction\x12\x0f\n\x07\x62\x61lance\x18\x04 \x01(\x04\"!\n\x0eTransactionReq\x12\x0f\n\x07tx_hash\x18\x01 \x01(\t\"\x99\x01\n\x0fTransactionResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12!\n\x02tx\x18\x03 \x01(\x0b\x32\x15.qrl.PlainTransaction\x12\x15\n\rconfirmations\x18\x04 \x01(\t\x12\x14\n\x0c\x62lock_number\x18\x05 \x01(\x04\x12\x19\n\x11\x62lock_header_hash\x18\x06 \x01(\t\"\x1d\n\nBalanceReq\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\";\n\x0b\x42\x61lanceResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07\x62\x61lance\x18\x03 \x01(\t\"\x11\n\x0fTotalBalanceReq\"@\n\x10TotalBalanceResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07\x62\x61lance\x18\x03 \x01(\t\"\x19\n\x06OTSReq\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"\x9b\x01\n\x07OTSResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x34\n\x14ots_bitfield_by_page\x18\x03 \x03(\x0b\x32\x16.qrl.OTSBitfieldByPage\x12\x1d\n\x15next_unused_ots_index\x18\x04 \x01(\x04\x12\x1e\n\x16unused_ots_index_found\x18\x05 \x01(\x08\"\x0b\n\tHeightReq\"9\n\nHeightResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0e\n\x06height\x18\x03 \x01(\x04\"\x1f\n\x08\x42lockReq\x12\x13\n\x0bheader_hash\x18\x01 \x01(\t\"H\n\tBlockResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x1e\n\x05\x62lock\x18\x03 \x01(\x0b\x32\x0f.qrl.PlainBlock\"(\n\x10\x42lockByNumberReq\x12\x14\n\x0c\x62lock_number\x18\x01 \x01(\x04\"\x1e\n\x10\x41\x64\x64ressFromPKReq\x12\n\n\x02pk\x18\x01 \x01(\t\"A\n\x11\x41\x64\x64ressFromPKResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\t\"\r\n\x0bNodeInfoReq\"\xc1\x01\n\x0cNodeInfoResp\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x17\n\x0fnum_connections\x18\x04 \x01(\t\x12\x17\n\x0fnum_known_peers\x18\x05 \x01(\t\x12\x0e\n\x06uptime\x18\x06 \x01(\x04\x12\x14\n\x0c\x62lock_height\x18\x07 \x01(\x04\x12\x17\n\x0f\x62lock_last_hash\x18\x08 \x01(\t\x12\x12\n\nnetwork_id\x18\t \x01(\t\"7\n\x13PlainGenesisBalance\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0f\n\x07\x62\x61lance\x18\x02 \x01(\x04\"5\n\x12PlainAddressAmount\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0e\n\x06\x61mount\x18\x02 \x01(\x04\"\xce\x07\n\x10PlainTransaction\x12\x13\n\x0bmaster_addr\x18\x01 \x01(\t\x12\x0b\n\x03\x66\x65\x65\x18\x02 \x01(\x04\x12\x12\n\npublic_key\x18\x03 \x01(\t\x12\x11\n\tsignature\x18\x04 \x01(\t\x12\r\n\x05nonce\x18\x05 \x01(\x04\x12\x18\n\x10transaction_hash\x18\x06 \x01(\t\x12\x13\n\x0bsigner_addr\x18\x07 \x01(\t\x12\x32\n\x08transfer\x18\x08 \x01(\x0b\x32\x1e.qrl.PlainTransaction.TransferH\x00\x12\x32\n\x08\x63oinbase\x18\t \x01(\x0b\x32\x1e.qrl.PlainTransaction.CoinBaseH\x00\x12;\n\tlatticePK\x18\n \x01(\x0b\x32&.qrl.PlainTransaction.LatticePublicKeyH\x00\x12\x30\n\x07message\x18\x0b \x01(\x0b\x32\x1d.qrl.PlainTransaction.MessageH\x00\x12,\n\x05token\x18\x0c \x01(\x0b\x32\x1b.qrl.PlainTransaction.TokenH\x00\x12=\n\x0etransfer_token\x18\r \x01(\x0b\x32#.qrl.PlainTransaction.TransferTokenH\x00\x12,\n\x05slave\x18\x0e \x01(\x0b\x32\x1b.qrl.PlainTransaction.SlaveH\x00\x1a-\n\x08Transfer\x12\x10\n\x08\x61\x64\x64rs_to\x18\x01 \x03(\t\x12\x0f\n\x07\x61mounts\x18\x02 \x03(\x04\x1a+\n\x08\x43oinBase\x12\x0f\n\x07\x61\x64\x64r_to\x18\x01 \x01(\t\x12\x0e\n\x06\x61mount\x18\x02 \x01(\x04\x1a:\n\x10LatticePublicKey\x12\x10\n\x08kyber_pk\x18\x01 \x01(\t\x12\x14\n\x0c\x64ilithium_pk\x18\x02 \x01(\t\x1a\x1f\n\x07Message\x12\x14\n\x0cmessage_hash\x18\x01 \x01(\t\x1ay\n\x05Token\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05owner\x18\x03 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\x04 \x01(\x04\x12\x31\n\x10initial_balances\x18\x05 \x03(\x0b\x32\x17.qrl.PlainAddressAmount\x1aH\n\rTransferToken\x12\x14\n\x0ctoken_txhash\x18\x01 \x01(\t\x12\x10\n\x08\x61\x64\x64rs_to\x18\x02 \x03(\t\x12\x0f\n\x07\x61mounts\x18\x03 \x03(\x04\x1a\x30\n\x05Slave\x12\x11\n\tslave_pks\x18\x01 \x03(\t\x12\x14\n\x0c\x61\x63\x63\x65ss_types\x18\x02 \x03(\rB\x11\n\x0ftransactionType\"\xdc\x01\n\x10PlainBlockHeader\x12\x13\n\x0bhash_header\x18\x01 \x01(\t\x12\x14\n\x0c\x62lock_number\x18\x02 \x01(\x04\x12\x19\n\x11timestamp_seconds\x18\x03 \x01(\x04\x12\x18\n\x10hash_header_prev\x18\x04 \x01(\t\x12\x14\n\x0creward_block\x18\x05 \x01(\x04\x12\x12\n\nreward_fee\x18\x06 \x01(\x04\x12\x13\n\x0bmerkle_root\x18\x07 \x01(\t\x12\x14\n\x0cmining_nonce\x18\x08 \x01(\r\x12\x13\n\x0b\x65xtra_nonce\x18\t \x01(\x04\"\x93\x01\n\nPlainBlock\x12%\n\x06header\x18\x01 \x01(\x0b\x32\x15.qrl.PlainBlockHeader\x12+\n\x0ctransactions\x18\x02 \x03(\x0b\x32\x15.qrl.PlainTransaction\x12\x31\n\x0fgenesis_balance\x18\x03 \x03(\x0b\x32\x18.qrl.PlainGenesisBalance2\xdb\x10\n\tWalletAPI\x12>\n\rAddNewAddress\x12\x15.qrl.AddNewAddressReq\x1a\x16.qrl.AddNewAddressResp\x12R\n\x17\x41\x64\x64NewAddressWithSlaves\x12\x1f.qrl.AddNewAddressWithSlavesReq\x1a\x16.qrl.AddNewAddressResp\x12>\n\rListAddresses\x12\x15.qrl.ListAddressesReq\x1a\x16.qrl.ListAddressesResp\x12>\n\rRemoveAddress\x12\x15.qrl.RemoveAddressReq\x1a\x16.qrl.RemoveAddressResp\x12=\n\x0eIsValidAddress\x12\x14.qrl.ValidAddressReq\x1a\x15.qrl.ValidAddressResp\x12>\n\rEncryptWallet\x12\x15.qrl.EncryptWalletReq\x1a\x16.qrl.EncryptWalletResp\x12\x35\n\nLockWallet\x12\x12.qrl.LockWalletReq\x1a\x13.qrl.LockWalletResp\x12;\n\x0cUnlockWallet\x12\x14.qrl.UnlockWalletReq\x1a\x15.qrl.UnlockWalletResp\x12G\n\x10GetRecoverySeeds\x12\x18.qrl.GetRecoverySeedsReq\x1a\x19.qrl.GetRecoverySeedsResp\x12>\n\rGetWalletInfo\x12\x15.qrl.GetWalletInfoReq\x1a\x16.qrl.GetWalletInfoResp\x12?\n\x10RelayTransferTxn\x12\x18.qrl.RelayTransferTxnReq\x1a\x11.qrl.RelayTxnResp\x12M\n\x17RelayTransferTxnBySlave\x12\x1f.qrl.RelayTransferTxnBySlaveReq\x1a\x11.qrl.RelayTxnResp\x12=\n\x0fRelayMessageTxn\x12\x17.qrl.RelayMessageTxnReq\x1a\x11.qrl.RelayTxnResp\x12K\n\x16RelayMessageTxnBySlave\x12\x1e.qrl.RelayMessageTxnBySlaveReq\x1a\x11.qrl.RelayTxnResp\x12\x39\n\rRelayTokenTxn\x12\x15.qrl.RelayTokenTxnReq\x1a\x11.qrl.RelayTxnResp\x12G\n\x14RelayTokenTxnBySlave\x12\x1c.qrl.RelayTokenTxnBySlaveReq\x1a\x11.qrl.RelayTxnResp\x12I\n\x15RelayTransferTokenTxn\x12\x1d.qrl.RelayTransferTokenTxnReq\x1a\x11.qrl.RelayTxnResp\x12W\n\x1cRelayTransferTokenTxnBySlave\x12$.qrl.RelayTransferTokenTxnBySlaveReq\x1a\x11.qrl.RelayTxnResp\x12\x39\n\rRelaySlaveTxn\x12\x15.qrl.RelaySlaveTxnReq\x1a\x11.qrl.RelayTxnResp\x12G\n\x14RelaySlaveTxnBySlave\x12\x1c.qrl.RelaySlaveTxnBySlaveReq\x1a\x11.qrl.RelayTxnResp\x12G\n\x10\x43hangePassphrase\x12\x18.qrl.ChangePassphraseReq\x1a\x19.qrl.ChangePassphraseResp\x12Y\n\x18GetTransactionsByAddress\x12\x1d.qrl.TransactionsByAddressReq\x1a\x1e.qrl.TransactionsByAddressResp\x12t\n!GetPaginatedTransactionsByAddress\x12&.qrl.PaginatedTransactionsByAddressReq\x1a\'.qrl.PaginatedTransactionsByAddressResp\x12;\n\x0eGetTransaction\x12\x13.qrl.TransactionReq\x1a\x14.qrl.TransactionResp\x12/\n\nGetBalance\x12\x0f.qrl.BalanceReq\x1a\x10.qrl.BalanceResp\x12>\n\x0fGetTotalBalance\x12\x14.qrl.TotalBalanceReq\x1a\x15.qrl.TotalBalanceResp\x12#\n\x06GetOTS\x12\x0b.qrl.OTSReq\x1a\x0c.qrl.OTSResp\x12,\n\tGetHeight\x12\x0e.qrl.HeightReq\x1a\x0f.qrl.HeightResp\x12)\n\x08GetBlock\x12\r.qrl.BlockReq\x1a\x0e.qrl.BlockResp\x12\x39\n\x10GetBlockByNumber\x12\x15.qrl.BlockByNumberReq\x1a\x0e.qrl.BlockResp\x12\x41\n\x10GetAddressFromPK\x12\x15.qrl.AddressFromPKReq\x1a\x16.qrl.AddressFromPKResp\x12\x32\n\x0bGetNodeInfo\x12\x10.qrl.NodeInfoReq\x1a\x11.qrl.NodeInfoRespb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qbitwallet_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ADDNEWADDRESSREQ']._serialized_start=37
  _globals['_ADDNEWADDRESSREQ']._serialized_end=94
  _globals['_ADDNEWADDRESSRESP']._serialized_start=96
  _globals['_ADDNEWADDRESSRESP']._serialized_end=161
  _globals['_ADDNEWADDRESSWITHSLAVESREQ']._serialized_start=163
  _globals['_ADDNEWADDRESSWITHSLAVESREQ']._serialized_end=256
  _globals['_ADDADDRESSFROMSEEDREQ']._serialized_start=258
  _globals['_ADDADDRESSFROMSEEDREQ']._serialized_end=295
  _globals['_ADDADDRESSFROMSEEDRESP']._serialized_start=297
  _globals['_ADDADDRESSFROMSEEDRESP']._serialized_end=367
  _globals['_LISTADDRESSESREQ']._serialized_start=369
  _globals['_LISTADDRESSESREQ']._serialized_end=387
  _globals['_LISTADDRESSESRESP']._serialized_start=389
  _globals['_LISTADDRESSESRESP']._serialized_end=456
  _globals['_REMOVEADDRESSREQ']._serialized_start=458
  _globals['_REMOVEADDRESSREQ']._serialized_end=493
  _globals['_REMOVEADDRESSRESP']._serialized_start=495
  _globals['_REMOVEADDRESSRESP']._serialized_end=543
  _globals['_VALIDADDRESSREQ']._serialized_start=545
  _globals['_VALIDADDRESSREQ']._serialized_end=579
  _globals['_VALIDADDRESSRESP']._serialized_start=581
  _globals['_VALIDADDRESSRESP']._serialized_end=643
  _globals['_ENCRYPTWALLETREQ']._serialized_start=645
  _globals['_ENCRYPTWALLETREQ']._serialized_end=683
  _globals['_ENCRYPTWALLETRESP']._serialized_start=685
  _globals['_ENCRYPTWALLETRESP']._serialized_end=733
  _globals['_UNLOCKWALLETREQ']._serialized_start=735
  _globals['_UNLOCKWALLETREQ']._serialized_end=772
  _globals['_UNLOCKWALLETRESP']._serialized_start=774
  _globals['_UNLOCKWALLETRESP']._serialized_end=821
  _globals['_LOCKWALLETREQ']._serialized_start=823
  _globals['_LOCKWALLETREQ']._serialized_end=838
  _globals['_LOCKWALLETRESP']._serialized_start=840
  _globals['_LOCKWALLETRESP']._serialized_end=885
  _globals['_GETRECOVERYSEEDSREQ']._serialized_start=887
  _globals['_GETRECOVERYSEEDSREQ']._serialized_end=925
  _globals['_GETRECOVERYSEEDSRESP']._serialized_start=927
  _globals['_GETRECOVERYSEEDSRESP']._serialized_end=1013
  _globals['_GETWALLETINFOREQ']._serialized_start=1015
  _globals['_GETWALLETINFOREQ']._serialized_end=1033
  _globals['_GETWALLETINFORESP']._serialized_start=1035
  _globals['_GETWALLETINFORESP']._serialized_end=1145
  _globals['_RELAYTRANSFERTXNREQ']._serialized_start=1148
  _globals['_RELAYTRANSFERTXNREQ']._serialized_end=1288
  _globals['_RELAYTRANSFERTXNBYSLAVEREQ']._serialized_start=1290
  _globals['_RELAYTRANSFERTXNBYSLAVEREQ']._serialized_end=1394
  _globals['_RELAYMESSAGETXNREQ']._serialized_start=1396
  _globals['_RELAYMESSAGETXNREQ']._serialized_end=1513
  _globals['_RELAYMESSAGETXNBYSLAVEREQ']._serialized_start=1515
  _globals['_RELAYMESSAGETXNBYSLAVEREQ']._serialized_end=1596
  _globals['_RELAYTOKENTXNREQ']._serialized_start=1599
  _globals['_RELAYTOKENTXNREQ']._serialized_end=1796
  _globals['_RELAYTOKENTXNBYSLAVEREQ']._serialized_start=1799
  _globals['_RELAYTOKENTXNBYSLAVEREQ']._serialized_end=1960
  _globals['_RELAYTRANSFERTOKENTXNREQ']._serialized_start=1963
  _globals['_RELAYTRANSFERTOKENTXNREQ']._serialized_end=2130
  _globals['_RELAYTRANSFERTOKENTXNBYSLAVEREQ']._serialized_start=2133
  _globals['_RELAYTRANSFERTOKENTXNBYSLAVEREQ']._serialized_end=2264
  _globals['_RELAYSLAVETXNREQ']._serialized_start=2267
  _globals['_RELAYSLAVETXNREQ']._serialized_end=2406
  _globals['_RELAYSLAVETXNBYSLAVEREQ']._serialized_start=2408
  _globals['_RELAYSLAVETXNBYSLAVEREQ']._serialized_end=2511
  _globals['_RELAYTXNRESP']._serialized_start=2513
  _globals['_RELAYTXNRESP']._serialized_end=2591
  _globals['_CHANGEPASSPHRASEREQ']._serialized_start=2593
  _globals['_CHANGEPASSPHRASEREQ']._serialized_end=2660
  _globals['_CHANGEPASSPHRASERESP']._serialized_start=2662
  _globals['_CHANGEPASSPHRASERESP']._serialized_end=2713
  _globals['_TRANSACTIONSBYADDRESSREQ']._serialized_start=2715
  _globals['_TRANSACTIONSBYADDRESSREQ']._serialized_end=2758
  _globals['_TRANSACTIONSBYADDRESSRESP']._serialized_start=2760
  _globals['_TRANSACTIONSBYADDRESSRESP']._serialized_end=2882
  _globals['_PAGINATEDTRANSACTIONSBYADDRESSREQ']._serialized_start=2884
  _globals['_PAGINATEDTRANSACTIONSBYADDRESSREQ']._serialized_end=2980
  _globals['_PAGINATEDTRANSACTIONSBYADDRESSRESP']._serialized_start=2983
  _globals['_PAGINATEDTRANSACTIONSBYADDRESSRESP']._serialized_end=3114
  _globals['_TRANSACTIONREQ']._serialized_start=3116
  _globals['_TRANSACTIONREQ']._serialized_end=3149
  _globals['_TRANSACTIONRESP']._serialized_start=3152
  _globals['_TRANSACTIONRESP']._serialized_end=3305
  _globals['_BALANCEREQ']._serialized_start=3307
  _globals['_BALANCEREQ']._serialized_end=3336
  _globals['_BALANCERESP']._serialized_start=3338
  _globals['_BALANCERESP']._serialized_end=3397
  _globals['_TOTALBALANCEREQ']._serialized_start=3399
  _globals['_TOTALBALANCEREQ']._serialized_end=3416
  _globals['_TOTALBALANCERESP']._serialized_start=3418
  _globals['_TOTALBALANCERESP']._serialized_end=3482
  _globals['_OTSREQ']._serialized_start=3484
  _globals['_OTSREQ']._serialized_end=3509
  _globals['_OTSRESP']._serialized_start=3512
  _globals['_OTSRESP']._serialized_end=3667
  _globals['_HEIGHTREQ']._serialized_start=3669
  _globals['_HEIGHTREQ']._serialized_end=3680
  _globals['_HEIGHTRESP']._serialized_start=3682
  _globals['_HEIGHTRESP']._serialized_end=3739
  _globals['_BLOCKREQ']._serialized_start=3741
  _globals['_BLOCKREQ']._serialized_end=3772
  _globals['_BLOCKRESP']._serialized_start=3774
  _globals['_BLOCKRESP']._serialized_end=3846
  _globals['_BLOCKBYNUMBERREQ']._serialized_start=3848
  _globals['_BLOCKBYNUMBERREQ']._serialized_end=3888
  _globals['_ADDRESSFROMPKREQ']._serialized_start=3890
  _globals['_ADDRESSFROMPKREQ']._serialized_end=3920
  _globals['_ADDRESSFROMPKRESP']._serialized_start=3922
  _globals['_ADDRESSFROMPKRESP']._serialized_end=3987
  _globals['_NODEINFOREQ']._serialized_start=3989
  _globals['_NODEINFOREQ']._serialized_end=4002
  _globals['_NODEINFORESP']._serialized_start=4005
  _globals['_NODEINFORESP']._serialized_end=4198
  _globals['_PLAINGENESISBALANCE']._serialized_start=4200
  _globals['_PLAINGENESISBALANCE']._serialized_end=4255
  _globals['_PLAINADDRESSAMOUNT']._serialized_start=4257
  _globals['_PLAINADDRESSAMOUNT']._serialized_end=4310
  _globals['_PLAINTRANSACTION']._serialized_start=4313
  _globals['_PLAINTRANSACTION']._serialized_end=5287
  _globals['_PLAINTRANSACTION_TRANSFER']._serialized_start=4838
  _globals['_PLAINTRANSACTION_TRANSFER']._serialized_end=4883
  _globals['_PLAINTRANSACTION_COINBASE']._serialized_start=4885
  _globals['_PLAINTRANSACTION_COINBASE']._serialized_end=4928
  _globals['_PLAINTRANSACTION_LATTICEPUBLICKEY']._serialized_start=4930
  _globals['_PLAINTRANSACTION_LATTICEPUBLICKEY']._serialized_end=4988
  _globals['_PLAINTRANSACTION_MESSAGE']._serialized_start=4990
  _globals['_PLAINTRANSACTION_MESSAGE']._serialized_end=5021
  _globals['_PLAINTRANSACTION_TOKEN']._serialized_start=5023
  _globals['_PLAINTRANSACTION_TOKEN']._serialized_end=5144
  _globals['_PLAINTRANSACTION_TRANSFERTOKEN']._serialized_start=5146
  _globals['_PLAINTRANSACTION_TRANSFERTOKEN']._serialized_end=5218
  _globals['_PLAINTRANSACTION_SLAVE']._serialized_start=5220
  _globals['_PLAINTRANSACTION_SLAVE']._serialized_end=5268
  _globals['_PLAINBLOCKHEADER']._serialized_start=5290
  _globals['_PLAINBLOCKHEADER']._serialized_end=5510
  _globals['_PLAINBLOCK']._serialized_start=5513
  _globals['_PLAINBLOCK']._serialized_end=5660
  _globals['_WALLETAPI']._serialized_start=5663
  _globals['_WALLETAPI']._serialized_end=7802
# @@protoc_insertion_point(module_scope)
